%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference: A Bayesian method for Deconvolution and Segmentation of
% Ultrasound Images based on GGD priors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Ningning Zhao (PhD student)
% University of Toulouse, IRIT/INP-ENSEEIHT
% date: Nov. 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% y:       blurred image
% psf_ref: known blurred kernel

clc
clear
close all
addpath 'funcs';
%% load image
%image='GGDrnd', 'simu'
image='simu';
input_images;

n0    = 500;
Nmc   = 1000;
steps = Nmc+n0;
if image=='real' %know ground truth of x: 1 (GGDrnd and simu), otherwise: 0(real)
    refx = 0;
else
    refx = 1;
end

ScriptParas;

isnr_x=zeros(steps,1);
%% Hybrid Gibbs Sampler
for ii=2:steps
    %% sample sig2n
    sig2n_sample(ii) = sample_sig2n_Gibbs(x_sample,psf_sample,y,0);
    
    %% sample the labels of RF image
    z_temp=z_sample;
    [z_b,ind_b] = label_sample_MRF_4neighbours(z_temp,x_sample,K,beta,xi_sample(:,ii-1),scale_sample(:,ii-1),mu_sample(:,ii-1),1);
    z_temp(ind_b)=z_b;
    [z_w,ind_w] = label_sample_MRF_4neighbours(z_temp,x_sample,K,beta,xi_sample(:,ii-1),scale_sample(:,ii-1),mu_sample(:,ii-1),2);
    z_temp(ind_w)=z_w;
    z_sample=z_temp;
    %     figure(30),imagesc(z_sample)
    %% sample shape and scale parameters
    for kk=1:K
        [xi_sample(kk,ii),accept_xi(kk,ii),wind_xi,walk_xi(kk,ii)]=...
            xi_sample_MH(xi_sample(kk,ii-1),mu_sample(kk,ii),scale_sample(kk,ii-1),x_sample,walk_xi(kk,ii-1),wind_xi,ii,n0,0,z_sample,kk);
        [scale_sample(kk,ii)] = scale_sample_Gibbs(x_sample,xi_sample(kk,ii),mu_sample(kk,ii),scale_sample(kk,ii-1),0,z_sample,kk);
    end
    %% sample x with HMC
    
    L=50+floor(rand*10);
    [x_sample,epsilon(ii),fenetre,accept_r(ii)]=...
        HMC_sample_v3(epsilon(ii-1),L,x_sample,z_sample,ii,n0,fenetre,y,psf_sample,sig2n_sample(ii),xi_sample(:,ii),scale_sample(:,ii),K);
    %% estimate of the PSF
    %% calculate MMSE estimates
    if ii<=n0
        x_hat=x_sample;
    else
        x_hat=(x_hat*(ii-1-n0)+x_sample)/(ii-n0);
    end
    
    for kk=1:K
        if ii>n0
            ind=find(z_sample==kk);
            temp=label_z(:,:,kk);
            temp(ind)=temp(ind)+1;
            label_z(:,:,kk)=temp;
        end
    end
    
    if refx==1
        isnr_x(ii) = ISNR_cal(y,refl,x_hat);
        %     figure(30),plot(isnr_x);
    end
    
    if mod(ii,100) == 1
        fprintf('No. Iter = %d, ISNR = %f\n',ii,isnr_x(ii));
    end
end

%% calculate the estimators
[~,ind]=max(label_z,[],3);
z_map=ind;

sig2n_est.mmse = mean(sig2n_sample(n0+1:end));
sig2n_est.sample=sig2n_sample;
for k=1:K
    xi_est.sample(k,:)=xi_sample(k,:);
    xi_est.mmse(k)=mean(xi_sample(k,n0+1:end));
    
    scale_est.sample(k,:)=scale_sample(k,:);
    scale_est.mmse(k)=mean(scale_sample(k,n0+1:end));
end
bmode_y=rf2bmode(y,10);
bmode_new=rf2bmode(x_hat,10);

% save([mat2str(clock) mat2str(image) '.mat']);
plot_results;

