function [x,err,f,xivec,sig2x,isnr_x] = GGD_deconv_label(y,P,z,K,refl,varargin)
 
if nargin < 6 || isempty(varargin{1})
    x0 = y; else x0 = varargin{1}; end % initial guess
if nargin < 7 || isempty(varargin{2})
    maxiter_em = 30; else maxiter_em = varargin{2}; end % max number of iterations
if nargin < 8 || isempty(varargin{3})
    tol = 1e-4; else tol = varargin{3}; end % tolerance
if nargin < 9 || isempty(varargin{4})
    verbose = 'on'; else verbose = varargin{4}; end
if nargin < 10 || isempty(varargin{5})
    sig2n = 0; else sig2n = varargin{5}; end
if nargin < 11 || isempty(varargin{6})
    sig2x = 0; else sig2x = varargin{6}; end


% if nargin < 7 || isempty(varargin{5})
%     c = 5e-4; else c = varargin{5}; end

% fixed parameters

if sig2n == 0   
    sig2n = estimate_noise_variance(y); % noise variance (wavelet estimate)
end

% if sig2x == 0
% if isreal(y) 
%     sig2y = var(y(:),1);
% else
%     sig2y = norm(y(:),2)^2/(2*numel(y));
% end
%     sig2x = abs( sig2y - sig2n ); % reflectivity variance 
% end


% internal parameters for CG algorithm
tol_CG = 1e-3;
maxit_CG = 30;
% c = 5e-4; % l1 norm smooth approximation
b = HTx_cal(P,y); % known term
stopping_rule = 3; % criterion used for stopping the cg

verbose_CG = 'on';
padding = 0;
interp_type = 'triang';
f = zeros(1,maxiter_em+1);
err = zeros(1,maxiter_em+1);
xivec = zeros(K,maxiter_em+1);

c=1e-8;
% iteration 0
x = x0;  
x_sub=cell(K,1);
xi=zeros(K,1);
%     sig2x(1)=43.5507;
%     sig2x(2)=1.2537;
    for k=1:K
    x_sub{k} = x(z==k);
    x_sub{k}=x_sub{k}-mean(x_sub{k});
    sig2x(k)=var(x_sub{k});
    xi(k) = xi_MlEst(x_sub{k},sig2x(k),c,2);
    end
xivec(:,1) = min([xi 2*ones(K,1)],[],2); % shape parameter

[f(1),D] = costFunc(x,y,P,c,xi,sig2n,z,K);
err(1) = 10000;

for kk = 1:maxiter_em
    
    [x] = ConjGradient2D(@(x)A(x,P,D),...
        b,@(x)JacobiPrecond(x,D),tol_CG,maxit_CG,x,stopping_rule,verbose_CG);
    
 isnr_x(kk) = ISNR_cal(y,refl,x);

    for k=1:K
    x_sub{k} = x(z==k);
    x_sub{k}=x_sub{k}-mean(x_sub{k});
    sig2x(k)=var(x_sub{k});   
    xivec(k,kk+1) = xi_MlEst(x_sub{k},sig2x(k),c,xi(k));
    end
%     xivec(1,kk+1)=1.8;
%     xivec(2,kk+1)=0.6;
    
    [f(kk+1), D] = costFunc(x,y,P,c,xivec(:,kk+1),sig2n,z,K);
    err(kk+1) = abs(f(kk+1)-f(kk))/abs(f(kk+1));
    if strcmp(verbose,'on');
%         disp(['error at ' num2str(kk) '-th iteration:' num2str(err(kk+1))]);
        disp(['iteration:' num2str(kk) ' ll: ' num2str(f(kk+1))]);
    end
    if err(kk+1) < tol
        disp(['convergence reached at ' num2str(kk) '-th iteration:']);
        err = err(1:kk+1);
        f = f(1:kk+1);
        xivec = xivec(:,1:kk+1);
        break
    end    
    
end

function y = A(x,P,D)
y = HTHx_cal(P,x) + D.*x;
% y = KTx_lin(conv2(x,P,'same'),P,padding,interp_type) + D.*x;

function y = JacobiPrecond(x,D) % jacobi preconditioner
y = x./(1+D);

function [f, D] = costFunc(x,y,P,c,xi,sig2n,z,K)
x_sub=cell(K,1);
sig2x=zeros(K,1);
% sig2x(1)=43.5507;
% sig2x(2)=1.2537;
n=ones(K,1);
for k=1:K
    x_sub{k} = x(z==k);
    x_sub{k}=x_sub{k}-mean(x_sub{k});
    sig2x(k)=var(x_sub{k}); 
    n(k)=numel(x_sub{k});
end
b = sqrt( sig2x.*gamma(1./xi)./gamma(3./xi) );
a = 1./(2*gamma(1+1./xi).*b);
lam = 1./(b.^xi);
% l2 term
e2 = Hx_cal(P,x) - y;
% e2 = conv2(x,P,'same') - y;
f2 = norm(e2(:),2)/(2*sig2n); 
% lp term
d=cell(K,1);
e1=cell(K,1);
D=z;
f1=ones(K,1);
for k=1:K
d{k} = sqrt( x_sub{k}.^2 + c);
e1{k} = d{k}.^xi(k);
f1(k) = sum(sum(e1{k})); % lp norm
D(D==k)=lam(k)*xi(k)./( d{k}.^(2-xi(k)));
end

f = f2 + sum(lam.*f1) - sum(n.*log(a)); % cost function value


function xi = xi_MlEst(x,sig2x,c,xi0)
% estimation of shape given the varianc e
n = numel(x);
A = @(p) sqrt(sig2x*gamma(1/p)/gamma(3/p)); % scale
b = @(p) .5/p^2 * (psi(1/p)-3*psi(3/p)); % derivative of log(A)
a = @(p) power(p,-2)*psi(1+1/p) + b(p); 
f = @(p) n*a(p) - sum( power(mod_normp(x,p,c)/A(p),p) .* ...
    ( log(mod_normp(x,p,c)/A(p)) + b(p)) );
xi = fzero(f,xi0);


function normx = mod_normp(x,p,c)
normx = power(x.^2+c,p/2);
