function y = KTx_lin(x,P,varargin)

% - double checked
if nargin < 3 || isempty(varargin{1})
    padding = 0; else padding = varargin{1}; end
if nargin < 4 || isempty(varargin{2})
    interp_type = 'triang'; else interp_type = varargin{2}; end

% -- product Kx for spatially variant blurs using linear interpolation
% suppose variations just in axial direction
[N M] = size(x);

if iscell(P)
    rax = P{1}.rax;
    rlat = P{1}.rlat;
    sax = P{1}.sax; % - stationarity regions
    slat = P{1}.slat; 
    % - masks used for interpolation
    [m_init m m_end] = buildMask([sax,slat],interp_type);
    xpad = padarray(x,[sax/2 slat/2],padding);
    ypad = zeros(N+sax+rax,M+slat+rlat);
    for ii = 1:numel(P) % -- vertical counter
        ycxwin = (ii-1)*sax + sax + 1; % -- center of the window             
        ycypad = (ii-1)*sax + rax/2 + sax + 1;
        xwin = xpad( ycxwin - sax : ycxwin + sax - 1, : );
        if ii == 1, wind = m_init; end
        if ii ~= 1 && ii ~= numel(P), wind = m; end
        if ii == numel(P), wind = m_end; end
        xwin = xwin .* wind;
        xwin = padarray(xwin,[rax/2 rlat/2]);
        pconjfft = P{ii}.pconjfft;
        ytemp = ifft2 (pconjfft .* fft2(xwin));
    %     ytemp = ytemp( rax/2:rax/2+2*sax-1 , rlat/2:rlat/2+2*slat-1 );
        ypad(ycypad-sax-rax/2: ycypad+sax+rax/2-1,:) = ...
            ypad(ycypad-sax-rax/2: ycypad+sax+rax/2-1,:) + ytemp;      
        debug();
    end    
    y = ypad( sax/2+rax/2+1:sax/2+rax/2+N,slat/2+rlat/2+1:slat/2+rlat/2+M );

else % single psf
    [rax rlat] = size(P); % padding psf
    rax = rax-1; rlat = rlat-1;
    xpad = padarray(x,[rax/2 rlat/2],padding);
    [nP mP] = size(xpad);
    psf = padarray( P , [nP-rax-1 mP-rlat-1], 'post' );
    psf = circshift( psf , [-floor(rax/2) -floor(rlat/2) ] );
    H = conj(fft2( psf ));
    y = ifft2(H.*fft2(xpad));
    y = y( rax/2+1:rax/2+N,rlat/2+1:rlat/2+M );
end

if isreal(x), y = real(y); end