function [xi_new,accept_xi,wind,walk_xi] = xi_sample_MH(xi_old,mu,scale,x,walk_xi,wind,ii,n0,wt,z,k)
%|sample \xi using Metropolis Hastings method with random walk
%|prior of xi: 1_{1,3}
%|2014/02/21
%|2014/02/24
%|2014/06/06

if wt==0
    if nargin~=11  
        return;
    end
    theta=x(z==k); 
elseif wt==1
    if nargin~=9   
        return;
    end
    theta=x;
end

N=numel(theta);

xi_cand = xi_old+walk_xi*randn;

% % negative log of the posterior distribution w.r.t. xi, and the
% likelihood function wrt xi is ggd
if isreal(theta)
    p_func=@(xi) N*log(2*scale^(1/xi)*gamma(1+1/xi))+norm(theta(:)-mu,xi)^xi/scale;
    p_cand=p_func(xi_cand);
    p_old=p_func(xi_old);
else
    p_func=@(xi) 2*N*log(2*scale^(1/xi)*gamma(1+1/xi))+(norm(real(theta(:))-mu,xi)^xi+norm(imag(theta(:))-mu,xi)^xi)/scale;
    p_cand=p_func(xi_cand);
    p_old=p_func(xi_old);    
end


if xi_cand<=3 && xi_cand>=0.01 
    accept= min(exp(p_old-p_cand),1);
else accept=0;
end
    
urn = rand;
if urn<=accept
    xi_new=xi_cand;
    label=1;
else
    xi_new=xi_old;
    label=0;
end

% adjust the random walk size 
nwind=30;
wind=[wind(1,2:nwind),label];
accept_xi=mean(wind);
if ii-1<=n0
    if  accept_xi<0.4
        walk_xi=walk_xi*(1-0.2);
    elseif accept_xi>0.8
        walk_xi= walk_xi*(1+0.2);
    end
elseif ii-1>n0
    if  accept_xi<0.4
        walk_xi=walk_xi*(1-0.1*exp(-0.01*(ii-n0)));
        elseif accept_xi>0.8
        walk_xi= walk_xi*(1+0.1*exp(-0.01*(ii-n0)));
    end
end
end




